<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

#namespace DeubertIt;

/**
 * @author Karsten Deubert <info@deubert.it>
 */
class DZF_Exception extends \Exception
{
    /**
     * @var mixed
     */
    public $debugData;

    /**
     * Filter sensitive data in passed array (should be $_SESSION).
     *
     * @param   array           $sessionInput
     * @return  array
     */
    public static function filterSession($sessionInput)
    {
        if (
            is_array($sessionInput)
            && array_key_exists('Zend_Auth', $sessionInput) && is_array($sessionInput['Zend_Auth'])
            && array_key_exists('storage', $sessionInput['Zend_Auth']) && is_array($sessionInput['Zend_Auth']['storage'])
        )
        {
            $sessionInput['Zend_Auth']['storage']['password'] = '***';
            $sessionInput['Zend_Auth']['storage']['password_salt'] = '***';
            $sessionInput['Zend_Auth']['storage']['salt'] = '***';
        }

        return $sessionInput;
    }

    /**
     * Convert exception object to the array.
     *
     * @param   \Exception           $exception
     * @return  array
     */
    public static function toCustomArray(\Exception $exception)
    {
        $traceData = [];

        foreach ($exception->getTrace() as $traceItem)
        {
            $traceItemData = $traceItem;
            $traceItemData['args'] = [];

            if (isset($traceItem['args']) && !empty($traceItem['args']))
            {
                foreach ($traceItem['args'] as $arg)
                {
                    $traceItemData['args'][] = self::_sanitizeKeys((array) $arg);
                }
            }

            if (empty($traceItemData['args']))
            {
                unset($traceItemData['args']);
            }

            $traceData[] = $traceItemData;
        }

        $sourceData = (array) $exception;
        unset($sourceData['trace']);
        unset($sourceData['previous']);

        $sourceData = self::_sanitizeKeys($sourceData);
        $sourceData['class'] = get_class($exception);
        $sourceData['trace'] = $traceData;

        $sourceData['previous'] = $exception->getPrevious()
            ? self::toCustomArray($exception->getPrevious())
            : null;

        return $sourceData;
    }

    /**
     * @param   array           $data
     * @return  array
     */
    protected static function _sanitizeKeys($data)
    {
        $result = [];

        foreach ($data as $key => $value)
        {
            if (strpos($key, '::__') !== false)
            {
                // skip "magic" methods
                continue;
            }

            // protected field starts with "\0*\0"
            // private field starts with "\0class_name\0"
            // for example (exception var_export):
            //   array(
            //     'debugData' => NULL,
            //     "\0" . '*' . "\0" . 'message' => 'some exception occurred',
            //     "\0" . 'Exception' . "\0" . 'string' => '',
            //     "\0" . '*' . "\0" . 'code' => 112233,
            //     "\0" . '*' . "\0" . 'file' => '/var/www/kpeye/app/modules/api/controllers/RequestlogController.php',
            //     "\0" . '*' . "\0" . 'line' => 38,
            //     "\0" . 'Exception' . "\0" . 'trace' =>
            //  ...

            $newKey = preg_replace("/\\0.*\\0/", '', $key);
            $result[$newKey] = $value;
        }

        return $result;
    }
}
