<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

#namespace DeubertIt;

/**
 * @author Karsten Deubert <info@deubert.it>
 */
class DZF_Validate_PostalCode extends \Zend_Validate_Abstract
{
	const INVALID = 'invalid';
	const TOOSHORT = 'toolong';
	const TOOLONG = 'tooshort';

    protected $_pattern = '/^[0-9]*$/ui'; //'/^[a-zA-Z0-9[:space:]\-]*$/ui';
    protected $_minLength = 5; //3;
    protected $_maxLength = 5; //10;

	protected $_messageTemplates = array(
		self::INVALID => "'%value%' enthält ungültige Zeichen.",
		self::TOOSHORT => "Die PLZ ist zu kurz.",
		self::TOOLONG => "Die PLZ ist zu lang."
	);

    public function setMinLength($minLength)
    {
        $this->_minLength = $minLength;
        return $this;
    }

    public function setMaxLength($maxLength)
    {
        $this->_maxLength = $maxLength;
        return $this;
    }

	public function isValid($value)
	{
		$this->_setValue($value);

        $length = mb_strlen($value);
        if ($length < $this->_minLength)
        {
            $this->_error(self::TOOSHORT);
            return false;
        }
        elseif ($length > $this->_maxLength)
        {
            $this->_error(self::TOOLONG);
            return false;
        }

        $regexValidator = new \Zend_Validate_Regex($this->_pattern);
        if (!$regexValidator->isValid($value))
        {
            $this->_error(self::INVALID);
            return false;
        }

		return true;
	}
}