<?php

class DZF_SettingsTests extends DZF_Helpers_TestCaseBase
{
    use TestHelpers;

    /** @var Zend_Cache_Backend_File */
    protected static $cache;

    /** @var DZF_Settings */
    protected static $settings;

    /** @var DZF_Settings_Source_Ini */
    protected static $source;

    public function __construct($name = null, array $data = [], $dataName = '')
    {
        if (!self::$cache) {
            $option = array(
                'group_key_1' => 'KPEYE_Bootstrap',
                'group_key_2' => 'ZendX/Application/Resource'
            );

            define('APPLICATION_PATH', realpath('') . '/tests/Helpers/Files');
            define('APPLICATION_ENV', 'production');

            self::$cache = new Zend_Cache_Backend_File($option);
            self::$settings = new DZF_Settings();

            $fileName = 'testIni.ini';
            $filePath = $this->getFilePath($fileName);
            self::$source = new DZF_Settings_Source_Ini($filePath);
        }

        parent::__construct($name, $data, $dataName);
    }

    public function testGetInstance()
    {
        $settings = DZF_Settings::getInstance();

        $this->assertNotNull($settings);
    }

    public function testSetCache()
    {
        $result = self::$settings->setCache(self::$cache);

        $this->assertNotNull($result);
    }

    public function testGetCache()
    {
        $result = self::$settings->getCache();

        $this->assertNotNull($result);
    }

    public function testSetCachedValue()
    {
        $groupKey = 'groupKey1';
        $paramName = 'paramName1';
        $value = 'value1';

        $result = self::$settings->setCachedValue($groupKey, $paramName, $value);

        $this->assertNotNull($result);
    }

    public function testGetCachedValue()
    {
        $groupKey = 'groupKey1';
        $paramName = 'paramName1';
        $expectedValue = 'value1';

        $value = self::$settings->getCachedValue($groupKey, $paramName);

        $this->assertEquals($expectedValue, $value);
    }

    public function testGetCachedValueNull()
    {
        $groupKey = 'groupKey2';
        $paramName = 'paramName2';
        $expectedValue = null;

        $value = self::$settings->getCachedValue($groupKey, $paramName);

        $this->assertEquals($expectedValue, $value);
    }

    public function testSetSource()
    {
        $result = self::$settings->setSource(self::$source);

        $this->assertNotNull($result);
    }

    public function testGetValueNull()
    {
        $groupKey = 'groupKey1';
        $paramName = 'paramName1';

        $result = self::$settings->getValue($groupKey, $paramName);

        $this->assertNull($result);
    }

    /**
     * @expectedException DZF_Exception
     * @expectedExceptionMessage Should be instance of DZF_Settings_Source of array of instances
     */
    public function testSetSourceException()
    {
        $result = self::$settings->setSource('source');

        $this->assertNotNull($result);
    }

    public function testClearCache()
    {
        $result = self::$settings->clearCache();

        $this->assertNotNull($result);
    }

    public function testGet()
    {
        $groupKey = 'groupKey1';
        $paramName = 'paramName1';

        $result = DZF_Settings::get($groupKey, $paramName);

        $this->assertNull($result);
    }
}
